#!/bin/bash
# ${copyright}
# ------------------------------------------------------------------------------
# # Install collectd 5.7.0 and some dependencies for plugins used in collectd.conf
# Run this script as root

trap "'echo: Uninstallation process cancelled by user.' && exit" SIGINT

KNOWN_DISTRIBUTION="(Debian|Ubuntu|RedHat|CentOS)"
DISTRIBUTION=$(lsb_release -d 2>/dev/null | grep -Eo $KNOWN_DISTRIBUTION)

# Detect root user
if [ $UID = "0" ]; then
    _sudo=''
else
    _sudo='sudo'
fi

if  [[ ! "$INSTALL_LOCATION" ]] ; then
    printf "\033[31mMissing required arguments - INSTALL_LOCATION.\033[0m\n"
    exit 1;
fi

if [ -n "$INSTALL_LOCATION" ]; then
    # assign splunk_home based on where user wants from the UI
    SPLUNK_FORWARDER_PATH="${INSTALL_LOCATION}/splunkforwarder"
fi

# All of the known constants that we have that we can set
STOP_COLLECTD_DEBIAN="service collectd stop"
STOP_COLLECTD_REDHAT="service collectd stop || $_sudo systemctl stop collectd.service"
STOP_COLLECTD_DARWIN="brew services stop collectd"
STOP_COLLECTD_SOLARIS="$_sudo svcadm disable cswcollectd"
UNINSTALL_COLLECTD_DEBIAN="$_sudo apt-get purge --auto-remove --yes collectd"
UNINSTALL_COLLECTD_REDHAT="$_sudo yum remove -y collectd"
UNINSTALL_COLLECTD_SUSE="$_sudo zypper remove -y collectd"
UNINSTALL_COLLECTD_DARWIN="brew remove collectd"
UNINSTALL_COLLECTD_SOLARIS="$_sudo /opt/csw/bin/pkgutil -r -y collectd"
COLLECTD_ASSET_PATH='/etc/collectd'

# Detect OS / Distribution
# We do not run this script to uninstall on EC2 instances, since we already have a separate configure
# page for it
if [ -f /etc/debian_version -o "$DISTRIBUTION" == "Debian" -o "$DISTRIBUTION" == "Ubuntu" ]; then
    UNINSTALL_COLLECTD=$UNINSTALL_COLLECTD_DEBIAN
    STOP_COLLECTD=$STOP_COLLECTD_DEBIAN
    COLLECTD_LIST=/etc/apt/sources.list.d/collectd-ci.list
    if [ -f "$COLLECTD_LIST" ] && grep -Fq "collectd-5.8" $COLLECTD_LIST ; then
        $_sudo sed -i '/collectd-5.8/d' $COLLECTD_LIST
        $_sudo apt-get update
    fi
elif [ -f /etc/redhat-release -o "$DISTRIBUTION" == "RedHat" -o "$DISTRIBUTION" == "CentOS" ]; then
    UNINSTALL_COLLECTD=$UNINSTALL_COLLECTD_REDHAT
    STOP_COLLECTD=$STOP_COLLECTD_REDHAT
elif [[ -f /etc/os-release && $(grep "^NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "openSUSE" ]]; then
    UNINSTALL_COLLECTD=$UNINSTALL_COLLECTD_SUSE
    STOP_COLLECTD=$STOP_COLLECTD_DEBIAN
else 
    OS=$(uname -s)
    if [ "$OS" = "SunOS" ]; then
        UNINSTALL_COLLECTD=$UNINSTALL_COLLECTD_SOLARIS
        STOP_COLLECTD=$STOP_COLLECTD_SOLARIS
    elif [ "$OS" = "Darwin" ]; then
        UNINSTALL_COLLECTD=$UNINSTALL_COLLECTD_DARWIN
        STOP_COLLECTD=$STOP_COLLECTD_DARWIN
    fi
fi

function uninstall_collectd_and_remove_assets {
    echo -e "\033[32m\nStep: Uninstalling collectd and removing all assets...\n\033[0m"
    # This uninstalls collectd
    $UNINSTALL_COLLECTD
    echo -e "\033[32mSuccessfully uninstalled collectd!\n\033[0m"
    # This removes all assets related to collectd
    $_sudo rm -rf "$COLLECTD_ASSET_PATH"
    echo -e "\033[32mSuccessfully removed all collectd assets!\n\033[0m"
}

function uninstall_splunkforwarder_and_remove_assets {
    echo -e "\033[32m\nStep: Uninstalling Splunk Forwarder and removing all assets...\n\033[0m"
    $_sudo rm -rf "$SPLUNK_FORWARDER_PATH"
    echo -e "\033[32mSuccessfully uninstalled Splunk Forwarder!\n\033[0m"
}

function stop_splunk_forwarder {
    echo -e "\033[32m\nStep: Stopping Splunk forwarder...\n\033[0m"
    if [ -d $SPLUNK_FORWARDER_PATH ]; then
        # This value is not needed, the next one is what we use
        status_message=$($_sudo "/$SPLUNK_FORWARDER_PATH/bin/splunk" status)
        status_value=$?
        if [ $status_value -eq "0" ]; then
            echo -e "\033[32mSplunk Forwarder is currently running.  Stopping...\n\033[0m"
            $_sudo "/$SPLUNK_FORWARDER_PATH/bin/splunk" stop
            echo -e "\033[32mSuccessfully stopped Splunk Forwarder!\n\033[0m"
        else
            echo -e "\033[32mSplunk Forwarder is already stopped. Skipping...\n\033[0m"
        fi
        # Disable boot start service 
        $_sudo /$SPLUNK_FORWARDER_PATH/bin/splunk disable boot-start
        echo -e "\033[32mSuccessfully disable Splunk boot start service!\n\033[0m"
    else
        echo -e "\033[32mSplunk Forwarder does not exist or has already been uninstalled. Skipping...\n\033[0m"
    fi
}

function stop_collectd_service {
    echo -e "\033[32m\nStep: Stopping the collectd service...\n\033[0m"
    $_sudo $STOP_COLLECTD
}

stop_collectd_service
stop_splunk_forwarder
uninstall_collectd_and_remove_assets
uninstall_splunkforwarder_and_remove_assets
